@@ -26,7 +26,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: enable_pps.c,v 1.28 1999/08/15 10:04:08 windl Exp $
+ * $Id: enable_pps.c,v 1.33 1999/10/03 14:43:57 windl Exp $
  */
 #include	<unistd.h>
 #include	<stdlib.h>
@@ -84,7 +84,7 @@
 #define	BITS_OFF(v, b)			((v) & ~(b))
 #define	BITS_ON(v, b)			((v) | (b))
 
-static	const char	id[] = "$Id: enable_pps.c,v 1.28 1999/08/15 10:04:08 windl Exp $";
+static	const char	id[] = "$Id: enable_pps.c,v 1.33 1999/10/03 14:43:57 windl Exp $";
 
 #define	BIT(n)	(1UL << (n))
 #define OPT_MONITOR		BIT(0)	/* perform monitor loop */
@@ -99,7 +99,7 @@ static	const char	id[] = "$Id: enable_pps.c,v 1.28 1999/08/15 10:04:08 windl Exp
 
 static	unsigned long	options = OPT_MONITOR | OPT_RESET;
 
-static	char	trigger	= 'A';		/* off | assert | clear | both */
+static	char		trigger	= 'A';	/* off | assert | clear | both */
 
 static	const	int	fd	= 0;	/* file descriptor of port (stdin) */
 static	pps_handle_t	ppshandle;	/* PPS handle (stdin) */
@@ -630,15 +630,17 @@ static	void	usage(void)
 	fprintf(stderr,
 		"Known options are:\n"
 		"\t-C\tdon't adjust time constant in monitoring loop\n"
+		"\t-EX\tEcho - set PPS API echo for `X' events (a|c)\n"
 		"\t-e\texit - do not enter monitoring loop\n"
 		"\t-F\tdon't adjust frequency in monitoring loop\n"
 		"\t-j\tmonitor jitter only\n"
-		"\t-k\tkeep settings (when entering monitoring loop)\n"
+		"\t-k\tkeep kernel flags (when entering monitoring loop)\n"
 #if defined(MOD_MICRO) && defined(MOD_NANO)
 		"\t-M\tenforce MOD_MICRO\n"
 		"\t-N\tenforce MOD_NANO\n"
 #endif
 		"\t-O\tdon't adjust offset in monitoring loop\n"
+		"\t-oXY\tSet PPS API offset for `X' events (a|c) to `Y'ns\n"
 		"\t-S\tdon't adjust status in monitoring loop\n"
 		"\t-t{0|a|c|b}[h]\ttrigger for event: off|assert|clear|both\n"
 		);
@@ -670,13 +672,30 @@ int	main(int argc, char *argv[])
 	       );
 	uname(&un);
 	printf(". Now running %s %s)\n", un.sysname, un.release);
-	while ( (ch = getopt(argc, argv, "CeFjkMNOSt:")) != -1 )
+	parm.mode = PPS_TSFMT_TSPEC;	/* set default flags */
+	while ( (ch = getopt(argc, argv, "CE:eFjkMNOo:St:")) != -1 )
 	{
 		switch ( ch )
 		{
 		case 'C':	/* don't adjust time constant */
 			options = BITS_ON(options, OPT_NO_TIMECONST);
 			break;
+		case 'E':
+			switch (optarg[0])
+			{
+			case 'a':
+				parm.mode |= PPS_ECHOASSERT;
+				break;
+			case 'c':
+				parm.mode |= PPS_ECHOCLEAR;
+				break;
+			default:
+				fprintf(stderr, "'-%c%s' is illegal\n",
+					ch, optarg);
+				usage();
+				exit(1);
+			}
+			break;
 		case 'e':	/* exit - don't enter monitoring loop */
 			options = BITS_OFF(options, OPT_MONITOR);
 			break;
@@ -702,6 +721,28 @@ int	main(int argc, char *argv[])
 		case 'S':	/* don't adjust time status */
 			options = BITS_ON(options, OPT_NO_STATUS);
 			break;
+		case 'o':
+			switch (optarg[0])
+			{
+			case 'a':
+				parm.mode |= PPS_OFFSETASSERT;
+				parm.assert_offset.tv_sec = 0;
+				parm.assert_offset.tv_nsec = strtol(optarg + 1,
+								    NULL, 10);
+				break;
+			case 'c':
+				parm.mode |= PPS_OFFSETCLEAR;
+				parm.clear_offset.tv_sec = 0;
+				parm.clear_offset.tv_nsec = strtol(optarg + 1,
+								   NULL, 10);
+				break;
+			default:
+				fprintf(stderr, "'-%c%s' is illegal\n",
+					ch, optarg);
+				usage();
+				exit(1);
+			}
+			break;
 		case 't':	/* select desired edge of the pulse */
 			trigger = optarg[0];
 			switch ( trigger )
@@ -712,9 +753,10 @@ int	main(int argc, char *argv[])
 			case 'b':
 				break;
 			default:
-				fprintf(stderr, "trigger '%s' is illegal\n",
-					optarg);
-				trigger = '0';
+				fprintf(stderr, "'-%c%s' is illegal\n",
+					ch, optarg);
+				usage();
+				exit(1);
 			}
 			if ( optarg[1] == 'h' )
 				trigger = toupper(trigger);
@@ -735,7 +777,6 @@ int	main(int argc, char *argv[])
 	ioctl(fd, TIOCMBIC, &stat);	/* power-on DCF77 receiver */
 	if ( time_pps_create(fd, &ppshandle) )
 		perror("time_pps_create()");
-	parm.mode = 0;			/* clear all flags */
 	switch ( trigger )
 	{
 	case 'A':
@@ -754,12 +795,18 @@ int	main(int argc, char *argv[])
 		parm.mode |= PPS_CAPTUREBOTH;
 		break;
 	}
-	if ( time_pps_setparams(ppshandle, &parm) != 0 )
-		perror("time_pps_setparams()");
 
-	fprintf(stderr,
-		"PPS API set up to detect '%c' events\n",
-		trigger);
+	if ( time_pps_setparams(ppshandle, &parm) != 0 )
+	{
+		perror("time_pps_setparams()");
+		fprintf(stderr, "handle=%d, mode=%#04x\n",
+			ppshandle, parm.mode);
+		/* continue, maybe just monitoring */
+	}
+	else
+		fprintf(stderr,
+			"PPS API set up (handle %d, mode %#04x)\n",
+			ppshandle, parm.mode);
 	memset(&ci, 0, sizeof(ci));
 	update_clock_info(&ci);
 	print_header();
